//******************************************************************************
// FILE:       	lcd.c
//              
// AUTHOR:     	CC Cox - SynchroSystems
//              
// PROJECT:    	SynchroChron
//              
// DESCRIPTION:	LCD driver functions
//					
// ICC430 only:	This driver uses the assert() macro to trap illegal parameters.
//				By default if assert() fails, it displays an error message and halts.
//				assert() can be disabled by compiling with NDEBUG defined.
//
// HISTORY:
//		$Log: lcd.c,v $
//		Revision 1.13  2007-08-20 16:58:48-04  chuck
//		Release
//
//		Revision 1.12  2007-08-20 16:38:58-04  chuck
//		Release
//
//		Revision 1.4 2006-01-13 chuck
//		Code review changes
//
//		Revision 1.3 2006-01-12 chuck
//		Code review release
//
//		Revision 1.0 2005-03-21 chuck
//		Initial revision
//
// Copyright (c) 2006 SynchroSystems Inc
// Use subject to terms of License Agreement at http://synchro.com/policies.html
//******************************************************************************

//------------------------------------------------------------------------------
// include files
//------------------------------------------------------------------------------
// system files
#include <string.h>					// string functions
#include <stdlib.h>					// standard library
#include <stdio.h>					// standard I/O
// project files
#include "msp430.h"					// processor-specific definitions
#include "macros.h"					// common macros
#include "lcd.h"					// driver interface
#include "lcd_font.h"				// font definition
#include <lcd_wiring.h>				// wiring configuration
#include "lcd_segs.h"				// segment map

//******************************************************************************
// PUBLIC DECLARATIONS
//******************************************************************************

//------------------------------------------------------------------------------
// ROM data
//------------------------------------------------------------------------------

//
// convert an integer(0..11) to dial segment mask (0x0000..0x0FFF)
//
const UINT16 LCD_DIAL_12[LCD_DIAL_12_LEN] =
{
	H12( 0),H12( 1),H12( 2),H12( 3),H12( 4),H12( 5),
	H12( 6),H12( 7),H12( 8),H12( 9),H12(10),H12(11)
};

//
// convert an integer (0..59) to dial hand mask  (0x0000..0x0FFF)
//
const UINT16 LCD_DIAL_60[LCD_DIAL_60_LEN] = 
{
	H60( 0),H60( 1),H60( 2),H60( 3),H60( 4),H60( 5),H60( 6),H60( 7),H60( 8),H60( 9), 
	H60(10),H60(11),H60(12),H60(13),H60(14),H60(15),H60(16),H60(17),H60(18),H60(19), 
	H60(20),H60(21),H60(22),H60(23),H60(24),H60(25),H60(26),H60(27),H60(28),H60(29), 
	H60(30),H60(31),H60(32),H60(33),H60(34),H60(35),H60(36),H60(37),H60(38),H60(39), 
	H60(40),H60(41),H60(42),H60(43),H60(44),H60(45),H60(46),H60(47),H60(48),H60(49), 
	H60(50),H60(51),H60(52),H60(53),H60(54),H60(55),H60(56),H60(57),H60(58),H60(59)
};

//
// convert an integer (0..99) to dial hand mask  (0x0000..0x0FFF)
//
const UINT16 LCD_DIAL_100[LCD_DIAL_100_LEN] = 
{
	H100( 0),H100( 1),H100( 2),H100( 3),H100( 4),H100( 5),H100( 6),H100( 7),H100( 8),H100( 9), 
	H100(10),H100(11),H100(12),H100(13),H100(14),H100(15),H100(16),H100(17),H100(18),H100(19), 
	H100(20),H100(21),H100(22),H100(23),H100(24),H100(25),H100(26),H100(27),H100(28),H100(29), 
	H100(30),H100(31),H100(32),H100(33),H100(34),H100(35),H100(36),H100(37),H100(38),H100(39), 
	H100(40),H100(41),H100(42),H100(43),H100(44),H100(45),H100(46),H100(47),H100(48),H100(49), 
	H100(50),H100(51),H100(52),H100(53),H100(54),H100(55),H100(56),H100(57),H100(58),H100(59), 
	H100(60),H100(61),H100(62),H100(63),H100(64),H100(65),H100(66),H100(67),H100(68),H100(69), 
	H100(70),H100(71),H100(72),H100(73),H100(74),H100(75),H100(76),H100(77),H100(78),H100(79), 
	H100(80),H100(81),H100(82),H100(83),H100(84),H100(85),H100(86),H100(87),H100(88),H100(89), 
	H100(90),H100(91),H100(92),H100(93),H100(94),H100(95),H100(96),H100(97),H100(98),H100(99)
};

//******************************************************************************
// PRIVATE DECLARATIONS
//******************************************************************************

//------------------------------------------------------------------------------
// constants
//------------------------------------------------------------------------------
#define	LCDMEM_BYTES	20					// size of LCDMEM in bytes
#define LAST_AREA		LCD_ICON_AREA		// last area
#define LAST_CHAR_AREA	LCD_PRIM_AREA		// last alphanumeric area
#define LAST_PUNCT_AREA	LCD_PRIM_PUNCT_AREA	// last punctuation area
#define LAST_CHAR_POS	((UINT8)LCD_PRIM_AREA + LCD_PRIM_LEN - 1) // last alphanumeric character position

//------------------------------------------------------------------------------
// ROM data
//------------------------------------------------------------------------------
static const UINT8 ID[] = "$Id: lcd.c,v 1.13 2007-08-20 16:58:48-04 chuck Exp chuck $";			// RCS ID string

//
// convert an integer (0..99) to a pair of ASCII characters
//
//lint -e{784}		NULL character intentionally truncated
static const UINT8 ASCII_INT[100][2] =
{
	"00", "01", "02", "03", "04", "05", "06", "07", "08", "09",
	"10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
	"20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
	"30", "31", "32", "33", "34", "35", "36", "37", "38", "39",
	"40", "41", "42", "43", "44", "45", "46", "47", "48", "49",
	"50", "51", "52", "53", "54", "55", "56", "57", "58", "59",
	"60", "61", "62", "63", "64", "65", "66", "67", "68", "69",
	"70", "71", "72", "73", "74", "75", "76", "77", "78", "79",
	"80", "81", "82", "83", "84", "85", "86", "87", "88", "89",
	"90", "91", "92", "93", "94", "95", "96", "97", "98", "99"
};

//------------------------------------------------------------------------------
// RAM data
//------------------------------------------------------------------------------
static UINT8 suppress_timer = 0;			// flash suppression timer
static UINT8 flash_lcdmem[LCDMEM_BYTES];	// flash mask
static const UINT8 *scroll_ptr = NULL;		// scroll buffer pointer
static INT8 scroll_idx = 0;					// scroll buffer index
static BOOL flash_enable = FALSE;			// flash enable flag
static BOOL scroll_enable = FALSE;			// scroll enable flag

//------------------------------------------------------------------------------
// functions
//------------------------------------------------------------------------------
static void show_segs(UINT16 segs, const SEGMENT *seg_ptr, BOOL flash);

//******************************************************************************
// PUBLIC FUNCTIONS
//******************************************************************************

//======================================================================
// control functions
//======================================================================

//------------------------------------------------------------------------------
//  FUNCTION:		LcdInit
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Initialize & energize LCD controller
//					Assumes P4SEL & P5SEL have been configured for LCD operation
//					Assumes BTCTL has been configured for LCD frequency
//------------------------------------------------------------------------------
void LcdInit(void)
{
	// use segment pins S0-S39
	// duty cycle = 4-mux
	// LCD timing, drive & segments on
	LCDCTL = LCDSG0_7 | LCD4MUX | LCDSON | LCDON; 
	LcdClear(); 
}

//------------------------------------------------------------------------------
//  FUNCTION:		LcdOn
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Turn on LCD timing generator & drive voltage
//------------------------------------------------------------------------------
void LcdOn(void)
{
	LCDCTL = LCDSG0_7 | LCD4MUX | LCDSON | LCDON;
}

//------------------------------------------------------------------------------
//  FUNCTION:		LcdOff
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Turn off LCD timing generator & drive voltage
//------------------------------------------------------------------------------
void LcdOff(void)
{
	LCDCTL = LCDSG0_7 | LCD4MUX | LCDSON; 
}

//------------------------------------------------------------------------------
//  FUNCTION:		LcdClear
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Clear entire display
//------------------------------------------------------------------------------
void LcdClear(void)
{
	// clear buffers
	(void)memset(LCDMEM, 0, LCDMEM_BYTES);
	(void)memset(flash_lcdmem, 0, LCDMEM_BYTES);
	// reset variables
	suppress_timer = 0;
	scroll_ptr = NULL;
	scroll_idx = 0;
	flash_enable = FALSE;
	scroll_enable = FALSE;
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdFlashSuppress
//              
//  PARAMETERS: 	duration	Duration in ticks (0 - 255)
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Stop flashing temporarily
//------------------------------------------------------------------------------
void LcdFlashSuppress(UINT8 duration)
{
	UINT8* lcdmem_ptr = LCDMEM;
	UINT8* flash_ptr = flash_lcdmem;
	UINT8 count;

	suppress_timer = duration;					// start timer
	
    // turn on all flashing segments
	for (count = LCDMEM_BYTES; count--; lcdmem_ptr++)
	{
		*lcdmem_ptr |= *(flash_ptr++);
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdFlashTimer
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Flash timer handler
//					Call periodically to flash display
//------------------------------------------------------------------------------
void LcdFlashTimer(void)
{
	UINT8 enable = 0;							// enable next cycle
	UINT8* lcdmem_ptr = LCDMEM;					// pointer to LCD memory
	UINT8* flash_ptr = flash_lcdmem;			// pointer to flash mask
	UINT8 i;									// loop counter
	
	if (suppress_timer != 0)
	{
		// flashing is suppressed
		suppress_timer--;
		flash_enable = TRUE;
	}
	else if (flash_enable)
	{
		// flashing is enabled, toggle all flashing segments 
		for (i = LCDMEM_BYTES; i--; lcdmem_ptr++, flash_ptr++)
		{
			enable |= *flash_ptr;				// remember if flashing
			*lcdmem_ptr ^= *flash_ptr;			// toggle flashing segments
		}
		flash_enable = (BOOL)(enable != 0);		// enable if still flashing
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdScrollTimer
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Scroll timer handler
//					Call periodically to scroll display
//------------------------------------------------------------------------------
void LcdScrollTimer(void)													
{
	const UINT8 *ptr;
	UINT8 len;
	
	if (scroll_enable)
	{
		assert(scroll_ptr != NULL);
		scroll_enable = TRUE;

		// update display
		if (scroll_idx < 0)
		{
			// show leading spaces
			LcdShowStr(scroll_ptr, LCD_PRIM_AREA, (UINT8)-scroll_idx, STEADY); 
		}
		else
		{
			// no leading spaces
			ptr = scroll_ptr + scroll_idx;
			len = (UINT8)strlen(ptr);
			LcdShowStr(ptr, LCD_PRIM_AREA, 0, STEADY);
			if (len < LCD_PRIM_LEN)
			{
				// show trailing spaces
				LcdShowChar(' ', LCD_PRIM_AREA, len, STEADY);
        		if (len == 0)
        		{
        			// message displayed, disable scrolling
        			scroll_ptr = NULL;
        			scroll_enable = FALSE;
        		}
			}
		}
		scroll_idx++;
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdIsScrolling
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	TRUE if text is actively scrolling
//					FALSE if text is not scrolling
//              
//  DESCRIPTION:	Indicates state of scrolling activity
//------------------------------------------------------------------------------
BOOL LcdIsScrolling(void)
{
	return (scroll_enable);
}

//======================================================================
// display functions
//======================================================================

//------------------------------------------------------------------------------
//  FUNCTION:		LcdShowSegs
//              
//  PARAMETERS: 	area		display area
//					pos			position of character in display area
//					segs		bit map of segments to display (0x0000 - 0x3FFF)
//					flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Control segments in single cell
//------------------------------------------------------------------------------
//lint -e{661}		out-of-bounds tested by assert()
void LcdShowSegs(UINT16 segs, LCD_AREA area, UINT8 pos, BOOL flash)
{
	assert(area <= LAST_AREA);
	
	switch (area)
	{
	case LCD_MODE_AREA:
		assert(pos < LCD_MODE_LEN);
		assert(segs <= LCD_ALPHA_ALL_SEGS);
		show_segs(segs, LCD_SEGS[(UINT8)area + pos], flash);
		break;
	case LCD_SEC_AREA:
		assert(pos < LCD_SEC_LEN);
		assert(segs <= LCD_ALPHA_ALL_SEGS);
		show_segs(segs, LCD_SEGS[(UINT8)area + pos], flash);
		break;
	case LCD_PRIM_AREA:
		assert(pos < LCD_PRIM_LEN);
		assert(segs <= LCD_ALPHA_ALL_SEGS);
		show_segs(segs, LCD_SEGS[(UINT8)area + pos], flash);
		break;
	case LCD_SEC_PUNCT_AREA:
		assert(segs <= LCD_PUNCT_ALL_SEGS);
		show_segs(segs, LCD_SEGS[(UINT8)area], flash);
		break;
	case LCD_PRIM_PUNCT_AREA:
		assert(pos < LCD_PRIM_PUNCT_LEN);
		assert(segs <= LCD_PUNCT_ALL_SEGS);
		show_segs(segs, LCD_SEGS[(UINT8)area + pos], flash);
		break;
	case LCD_DIAL_AREA:
		assert(segs <= LCD_DIAL_ALL_SEGS);
		show_segs(segs, LCD_SEGS[(UINT8)area], flash);
		break;
	case LCD_ICON_AREA:
		assert(pos < LCD_ICON_LEN);
		assert(segs <= LCD_ICON_ALL_SEGS);
		show_segs(segs, LCD_SEGS[(UINT8)area + pos], flash);
		break;
	default:
		// ignore unknown areas
		break;
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:		LcdShowChar
//              
//  PARAMETERS: 	area		display area
//					pos			position of character in display area
//  			 	c			ASCII character to display (0x20 - 0x7F)
//					flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display a single ASCII character
//------------------------------------------------------------------------------
//lint -e{661} out-of-bounds tested by assert()
void LcdShowChar(UINT8 c, LCD_AREA area, UINT8 pos, BOOL flash)
{
	assert(area <= LAST_PUNCT_AREA);
	assert(c >= ' ');
	assert(c <= DEL);
	
	switch (area)
	{
	case LCD_MODE_AREA:
		assert(pos < LCD_MODE_LEN + LCD_SEC_LEN + LCD_PRIM_LEN);
		show_segs(FONT_ASCII[c - ' '], LCD_SEGS[(UINT8)area + pos], flash);
		break;
	case LCD_SEC_AREA:
		assert(pos < LCD_SEC_LEN + LCD_PRIM_LEN);
		show_segs(FONT_ASCII[c - ' '], LCD_SEGS[(UINT8)area + pos], flash);
		break;
	case LCD_PRIM_AREA:
		assert(pos < LCD_PRIM_LEN);
		show_segs(FONT_ASCII[c - ' '], LCD_SEGS[(UINT8)area + pos], flash);
		break;
	case LCD_SEC_PUNCT_AREA:
	   	assert((c == '.') || (c == '-') || (c == ':') || (c == ' '));			
		switch (c)
		{
		case '-':
			show_segs(SEGA,        LCD_SEGS[(UINT8)area], flash);
			break;
		case '.':
			show_segs(SEGB,        LCD_SEGS[(UINT8)area], flash);
			break;
		case ':':
			show_segs(SEGA | SEGB, LCD_SEGS[(UINT8)area], flash);
			break;
		default:
			show_segs(0,           LCD_SEGS[(UINT8)area], flash);
			break;
		}
		break;
	case LCD_PRIM_PUNCT_AREA:
		assert(pos < LCD_PRIM_PUNCT_LEN);
	   	assert((c == '.') || (c == '-') || (c == ':') || (c == ' '));											
		switch (c)
		{
		case '-':
			show_segs(SEGA,        LCD_SEGS[(UINT8)area + pos], flash);
			break;
		case '.':
			show_segs(SEGB,        LCD_SEGS[(UINT8)area + pos], flash);
			break;
		case ':':
			show_segs(SEGA | SEGB, LCD_SEGS[(UINT8)area + pos], flash);
			break;
		default:
			show_segs(0,           LCD_SEGS[(UINT8)area + pos], flash);
			break;
		}
		break;
	case LCD_ICON_AREA:
		assert(pos < LCD_ICON_LEN);
	   	assert((c == '0') || (c == '1') || (c == ' '));											
		switch (c)
		{
		case '1':
			show_segs(SEGA,        LCD_SEGS[(UINT8)area + pos], flash);
			break;
		default:
			show_segs(0,           LCD_SEGS[(UINT8)area + pos], flash);
			break;
		}
		break;
	case LCD_DIAL_AREA:
		// fallthrough
	default:
		// ignore other areas
		break;
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowStr
//              
//  PARAMETERS: 	area		display area
//					pos			starting position on display (0 - 5)
//  			 	s			pointer to string to display
//					flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display a string
//------------------------------------------------------------------------------
void LcdShowStr(const UINT8* s, LCD_AREA area, UINT8 pos, BOOL flash)
{
	assert(area <= LAST_CHAR_AREA);
	assert(pos < LCD_PRIM_LEN);

	// display all characters in string
	while ((*s != '\0') && (pos < LCD_PRIM_LEN))
	{
		LcdShowChar(*s, area, pos, flash);	// display character
		s++;								// point to next character
		pos++;								// next position
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowStrN
//              
//  PARAMETERS: 	area		display area
//					pos			starting position on display (0 - 5)
//  			 	s			pointer to string to display
//					flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display a fixed-length string
//------------------------------------------------------------------------------
void LcdShowStrN(const UINT8* s, LCD_AREA area, UINT8 pos, UINT8 len, BOOL flash)
{
	assert(area <= LAST_CHAR_AREA);
	assert(pos < LCD_PRIM_LEN);
	assert((len > 0) && (len <= LCD_PRIM_LEN));

	// display all characters in string
	while ((*s != '\0') && (pos < LCD_PRIM_LEN) && (len > 0))
	{
		LcdShowChar(*s, area, pos, flash);	// display character
		s++;								// point to next character
		pos++;								// next position
		len--;
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowScroll
//              
//  PARAMETERS: 	s			String to display
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display a scrolling string in primary area
//------------------------------------------------------------------------------
void LcdShowScroll(const UINT8* s)
{
	assert(strlen(s) > LCD_PRIM_LEN);
	
	// clear primary display area
	LcdShowChar(' ', LCD_PRIM_PUNCT_AREA, 0, FALSE);					
	LcdShowChar(' ', LCD_PRIM_PUNCT_AREA, 1, FALSE);					
	LcdShowStr("      ", LCD_PRIM_AREA, 0, FALSE);				
	// initialize scrolling variables
	scroll_ptr = s;							
	scroll_idx = -5;
	scroll_enable = TRUE;			
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowUDec2
//              
//  PARAMETERS: 	area		area of display to display in
//					pos			Starting position on display (0 - 5)
//  			 	value		integer value to display (0 - 99)
//					lead_zero	Leading zero enable
//					flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display a two-digit unsigned decimal integer
//					with optional leading zero
//------------------------------------------------------------------------------
//lint -e{661} -e{662} out-of-bounds tested by assert()
void LcdShowUDec2(UINT8 value, LCD_AREA area, UINT8 pos, BOOL flash, BOOL lead_zero)
{
	assert(area <= LAST_CHAR_AREA);
	assert(pos <= LCD_PRIM_LEN);
	assert(value <= 99);

	// display most significant digit
	if (!lead_zero && (value < 10))
	{
		// suppress leading zero
		LcdShowChar(' ', area, pos++, flash);					
	}
	else
	{
		// display digit
    	LcdShowChar(ASCII_INT[value][0], area, pos++, flash);
	}
	// display least significant digit	
	LcdShowChar(ASCII_INT[value][1], area, pos, flash);		
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowUDec4
//              
//  PARAMETERS: 	area		area of display to display in
//					pos			Starting position on display (0 - 5)
//  			 	value		Byte integer value to display (0 - 9999)
//					lead_zero	Leading zero enable
//					flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display a four-digit unsigned decimal integer
//					with optional leading zeros
//------------------------------------------------------------------------------
void LcdShowUDec4(UINT16 value, LCD_AREA area, UINT8 pos, BOOL flash, BOOL lead_zero)
{
	UINT16 msd;						// most significant digits
	UINT16 lsd;						// least significant digits
	
	assert(area <= LAST_CHAR_AREA);
	assert(pos <= LCD_PRIM_LEN);
	assert(value <= 9999);
	
	// display most significant digit pair
	msd = value / 100;					
	lsd = value - (msd * 100);			
	if (!lead_zero && (msd == 0))
	{
		// suppress leading zeros
		LcdShowChar(' ', area, pos++, flash);					
		LcdShowChar(' ', area, pos++, flash);					
		// display least significant digit pair
   		LcdShowUDec2((UINT8)lsd, area, pos, flash, LEAD_SPACE);
	}
	else
	{
		// display digits
    	LcdShowUDec2((UINT8)msd, area, pos, flash, lead_zero);
		pos += 2;
		// display least significant digit pair
   		LcdShowUDec2((UINT8)lsd, area, pos, flash, LEAD_ZERO);
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowDialCW
//              
//  PARAMETERS: 	mask_ptr	pointer to dial segment mask
//  			 	flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display dial segments, rotate mask clockwise
//------------------------------------------------------------------------------
void LcdShowDialCW(UINT16* mask_ptr, BOOL flash)
{
	// show current mask
	show_segs(*mask_ptr, LCD_SEGS[LCD_DIAL_AREA], flash);
	// rotate mask clockwise
	*mask_ptr <<= 1;
	// handle wrap-around
	if (BITTST(*mask_ptr, LCD_DIAL_ALL_SEGS + 1))
	{
		BITSET(*mask_ptr, 1);
		BITCLR(*mask_ptr, LCD_DIAL_ALL_SEGS + 1);
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowDialCCW
//              
//  PARAMETERS: 	mask_ptr	pointer to dial segment mask
//  			 	flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display dial segments, rotate mask counter-clockwise
//------------------------------------------------------------------------------
void LcdShowDialCCW(UINT16* mask_ptr, BOOL flash)
{
	// show current mask
	show_segs(*mask_ptr, LCD_SEGS[LCD_DIAL_AREA], flash);
	// handle wrap-around
	if (BITTST(*mask_ptr, 1)) 
	{
		BITSET(*mask_ptr, LCD_DIAL_ALL_SEGS + 1);
	}
	// rotate mask counter-clockwise
	*mask_ptr >>= 1;
}

//======================================================================
// stdio functions
//======================================================================

//------------------------------------------------------------------------------
//  FUNCTION:   	putchar
//              
//  PARAMETERS: 	c		character to display
//              
//  RETURNS:    	character displayed
//              
//  DESCRIPTION:	stdio stream interface to LCD, overrides library function
//					display characters in mode, secondary, then primary display area
//					CR or LF reset index to leftmost position mode area
//------------------------------------------------------------------------------
int putchar(PUTCHAR_ARG_TYPE c)
{
	static UINT8 idx = 0;
	
	if ((c == '\n') || (c == '\r')) 
	{
		idx = 0;							// reset index left mode char
	}
	else if (idx <= LAST_CHAR_POS) 
	{
		c -= ' ';							// convert to font index
		show_segs(FONT_ASCII[c], LCD_SEGS[idx], STEADY); // show char
		idx++;								// next character position
	}
	return (c);
}

//======================================================================
// utility functions
//======================================================================

//------------------------------------------------------------------------------
//  FUNCTION:   	LcdShowError
//              
//  PARAMETERS: 	mode_ptr	pointer to string to display in mode area
//  			 	str_ptr		pointer to string to display in primary area
//					num			integer to display in secondary area
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display error message & halt execution
//------------------------------------------------------------------------------
void LcdShowError(const UINT8* mode_ptr, const UINT8* str_ptr, UINT16 num)
{
	// show error message
	LcdClear();
	LcdShowStr(mode_ptr, LCD_MODE_AREA, 0, FLASH);			// error type
	LcdShowUDec4(num, LCD_SEC_AREA, 0, LEAD_ZERO, STEADY);	// show number in secondary
	LcdShowStr(str_ptr, LCD_PRIM_AREA, 0, STEADY); 			// show string in primary
	// stop the program
	HALT();
}

//******************************************************************************
// PRIVATE FUNCTIONS
//******************************************************************************

//------------------------------------------------------------------------------
//  FUNCTION:		show_segs
//              
//  PARAMETERS: 	seg_ptr		Pointer to array of segment definitions
//					segs		Bit map of segments to display
//					flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	low-level segment display function
//------------------------------------------------------------------------------
static void show_segs(UINT16 segs, const SEGMENT *seg_ptr, BOOL flash)
{
	UINT16 mask = 1;							// segment mask

	if (!flash)
	{
    	// display steady segments
    	do	
    	{
    		if (segs & mask)
    		{
    			// segment on
    			BITSET(LCDMEM[seg_ptr->offset], seg_ptr->mask);		
    		}
    		else
    		{
    			// segment off
    			BITCLR(LCDMEM[seg_ptr->offset], seg_ptr->mask);			
    		}
    		// segment steady
    		BITCLR(flash_lcdmem[seg_ptr->offset], seg_ptr->mask);	
    		mask <<= 1;							// mask for next segment
    		seg_ptr++;							// point to next segment
    	}
    	while (seg_ptr->offset != END_MARK);
	}
	else
	{
    	// display flashing segments
    	do	
    	{
      		if (segs & mask)
    		{
    			// segment on & flashing
    			BITSET(LCDMEM[seg_ptr->offset], seg_ptr->mask);		
    			BITSET(flash_lcdmem[seg_ptr->offset], seg_ptr->mask);
    		}
    		else
    		{
    			// segment off & steady
    			BITCLR(LCDMEM[seg_ptr->offset], seg_ptr->mask);			
    			BITCLR(flash_lcdmem[seg_ptr->offset], seg_ptr->mask);	
    		}
    		mask <<= 1;							// mask for next segment
    		seg_ptr++;							// point to next segment
    	}
    	while (seg_ptr->offset != END_MARK);
    	
    	flash_enable = TRUE;					// enable flashing
	}
}

