//******************************************************************************
// FILE:       	lcd_segs.h
//              
// AUTHOR:     	CC Cox - SynchroSystems
//              
// PROJECT:    	SynchroChron
//              
// DESCRIPTION:	LCD segment map header file
//
// HISTORY:
//		$Log: lcd_segs.h,v $
//		Revision 1.4  2006-01-26 14:41:22-05  chuck
//		<>
//
//		Revision 1.3 2006-01-13 chuck
//		Code review changes
//
//		Revision 1.2 2006-01-12 chuck
//		Code review release
//
//		Revision 1.0 2005-03-11 chuck
//		Initial revision
//
// Copyright (c) 2006 SynchroSystems Inc
//******************************************************************************
#ifndef __LCD_SEGS_H
#define __LCD_SEGS_H

//******************************************************************************
// PRIVATE DECLARATIONS
//******************************************************************************

//------------------------------------------------------------------------------
// constants
//------------------------------------------------------------------------------
#define MAX_SEGS	14				// max segments per cell
#define END_MARK	0xFFu			// end of cell marker 

//------------------------------------------------------------------------------
// macros
//------------------------------------------------------------------------------
#define	END			{END_MARK, END_MARK}	// end of cell marker
// Convert segment (0-39) and common (0-3) into an LCDMEM offset and bit mask.
#define SEG(seg, com)	{(seg) / 2, 1 << ((com) + 4 * ((seg) % 2))}
//------------------------------------------------------------------------------
// typedefs
//------------------------------------------------------------------------------
// define the display memory bit for a single segment
typedef struct
{
	UINT8 offset;						// LCDMEM byte offset
	UINT8 mask;							// bit mask
} SEGMENT;

//------------------------------------------------------------------------------
// ROM data
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// LCD segment map table
//
// Converts cell positions and segments into LCDMEM addresses and bit masks.
//
// Each line defines a single cell.
// Each line consists of a list of segments, terminated by an end marker.
// Alphanumeric characters consist of 14 segments.
// Numeric digits consist of 7 segments.
// Each punctuation mark consists of two segments.
// The dial consists of 12 hands, each consisting of one segment.
// There are 2 icons, consisting of one segment each.
//------------------------------------------------------------------------------
static const SEGMENT LCD_SEGS[][MAX_SEGS + 1] = 
{
	//--------------------------------------------------------------------------
	// mode display
	//
	// 2 small alphanumeric characters with 14 segments each
	//
	// segment map:
	//   A           B           C           D           E           F           G           H           J           K           M           N           P           Q
	//--------------------------------------------------------------------------
	{SEG(S37,C0),SEG(S36,C0),SEG(S36,C2),SEG(S38,C3),SEG(S39,C3),SEG(S39,C0),SEG(S39,C1),SEG(S38,C0),SEG(S38,C1),SEG(S37,C1),SEG(S37,C2),SEG(S37,C3),SEG(S38,C2),SEG(S39,C2),END},
	{SEG(S34,C0),SEG(S33,C0),SEG(S33,C3),SEG(S35,C3),SEG(S36,C3),SEG(S36,C1),SEG(S35,C1),SEG(S35,C0),SEG(S34,C1),SEG(S33,C1),SEG(S33,C2),SEG(S34,C3),SEG(S34,C2),SEG(S35,C2),END},	
	//--------------------------------------------------------------------------
	// secondary display
	//
	// 4 small numeric digits with 7 segments each
	//
	// segment map:
	//   A           B           C           D           E           F           G           
	//--------------------------------------------------------------------------
	{SEG(S28,C0),SEG(S28,C1),SEG(S28,C2),SEG(S28,C3),SEG(S29,C2),SEG(S29,C0),SEG(S29,C1),END,END,END,END,END,END,END,END},													
	{SEG(S26,C0),SEG(S26,C1),SEG(S26,C2),SEG(S26,C3),SEG(S27,C2),SEG(S27,C0),SEG(S27,C1),END,END,END,END,END,END,END,END},														
	{SEG(S24,C0),SEG(S24,C1),SEG(S24,C2),SEG(S24,C3),SEG(S25,C2),SEG(S25,C0),SEG(S25,C1),END,END,END,END,END,END,END,END},														
	{SEG(S22,C0),SEG(S22,C1),SEG(S22,C2),SEG(S22,C3),SEG(S23,C2),SEG(S23,C0),SEG(S23,C1),END,END,END,END,END,END,END,END},													
	//--------------------------------------------------------------------------
	// primary display
	//
	// 6 large alphanumeric characters with 14 segments each
	//
	// segment map:
	//   A           B           C           D           E           F           G           H           J           K           M           N           P           Q
	//--------------------------------------------------------------------------
	{SEG(S02,C3),SEG(S03,C3),SEG(S03,C1),SEG(S01,C0),SEG(S00,C0),SEG(S00,C3),SEG(S00,C2),SEG(S01,C3),SEG(S01,C2),SEG(S02,C2),SEG(S02,C1),SEG(S02,C0),SEG(S01,C1),SEG(S00,C1),END},	
	{SEG(S05,C3),SEG(S06,C3),SEG(S06,C0),SEG(S04,C0),SEG(S03,C0),SEG(S03,C2),SEG(S04,C2),SEG(S04,C3),SEG(S05,C2),SEG(S06,C2),SEG(S06,C1),SEG(S05,C0),SEG(S05,C1),SEG(S04,C1),END},	
	{SEG(S09,C3),SEG(S10,C3),SEG(S10,C1),SEG(S08,C0),SEG(S07,C0),SEG(S07,C3),SEG(S07,C2),SEG(S08,C3),SEG(S08,C2),SEG(S09,C2),SEG(S09,C1),SEG(S09,C0),SEG(S08,C1),SEG(S07,C1),END},	
	{SEG(S12,C3),SEG(S13,C3),SEG(S13,C0),SEG(S11,C0),SEG(S10,C0),SEG(S10,C2),SEG(S11,C2),SEG(S11,C3),SEG(S12,C2),SEG(S13,C2),SEG(S13,C1),SEG(S12,C0),SEG(S12,C1),SEG(S11,C1),END},
	{SEG(S17,C3),SEG(S18,C3),SEG(S18,C1),SEG(S16,C0),SEG(S15,C0),SEG(S15,C3),SEG(S15,C2),SEG(S16,C3),SEG(S16,C2),SEG(S17,C2),SEG(S17,C1),SEG(S17,C0),SEG(S16,C1),SEG(S15,C1),END},	
	{SEG(S20,C3),SEG(S21,C3),SEG(S21,C0),SEG(S19,C0),SEG(S18,C0),SEG(S18,C2),SEG(S19,C2),SEG(S19,C3),SEG(S20,C2),SEG(S21,C2),SEG(S21,C1),SEG(S20,C0),SEG(S20,C1),SEG(S19,C1),END},	
	//--------------------------------------------------------------------------
	// punctuation marks
	//
	// there are 3 punctuation marks, with 2 segments each
	//
	// segment map:
	//   dash        period
	//--------------------------------------------------------------------------
	// secondary display: 1 small punctuation mark
	{SEG(S25,C3),SEG(S23,C3),END,END,END,END,END,END,END,END,END,END,END,END,END},																			
	// primary display: 2 large punctuation marks
	{SEG(S14,C3),SEG(S14,C2),END,END,END,END,END,END,END,END,END,END,END,END,END},																		
	{SEG(S14,C1),SEG(S14,C0),END,END,END,END,END,END,END,END,END,END,END,END,END},											
	//--------------------------------------------------------------------------
	// dial hands
	//
	// the dial has 12 segments, one for each hand
	//
	// segment map:
	//   12          1           2           3           4           5           6           7           8           9           10          11
	//--------------------------------------------------------------------------
	{SEG(S30,C0),SEG(S30,C1),SEG(S30,C2),SEG(S30,C3),SEG(S29,C3),SEG(S27,C3),SEG(S32,C2),SEG(S32,C3),SEG(S31,C3),SEG(S31,C2),SEG(S31,C1),SEG(S31,C0),END,END,END},
	//--------------------------------------------------------------------------
	// icons
	//
	// there are 2 icons, with one segment each
	//--------------------------------------------------------------------------
	{SEG(S32,C0),END,END,END,END,END,END,END,END,END,END,END,END,END,END},					// alarm (exclamation mark)
	{SEG(S32,C1),END,END,END,END,END,END,END,END,END,END,END,END,END,END}					// bell																		
};

#endif	// __LCD_SEGS_H
